# This Python file uses the following encoding: utf-8
from __future__ import with_statement 
import math, vcVector, vcMatrix
import os.path
import datetime
from PIL import Image
try:from vcScript import *
except:pass
try:from vcCommand import *
except:pass
from collections import defaultdict

LINECHANGE = '\n'

def sanitizefilename(fname):
  suspiciouscharacters = [r'/', '\\' ,'#',':',';','!','','%','&','?','^','*',' ','\t']
  for c in suspiciouscharacters:
    fname = fname.replace(c,'_')
  return fname

def export_obj(app, node, materials, material_file, scale, filename, mat_mode, mat_csv, mat_dict, SEP):
  new_materials = {}
  dirM = vcMatrix.new()
  #filename = sanitizefilename(filename)
  basename, extension = os.path.splitext(os.path.basename(filename))
  dirname = os.path.dirname(filename)
  if not material_file:
    mtlfilename = dirname +'\\'+ "VC_to_Blender_materials.mtl"
  ii = 1
  hasgeometry = False
  for gs in node.Geometry.GeometrySets:
    if gs.Type == VC_TRIANGLESET:
      if gs.PointCount > 0:
        hasgeometry = True
        break
  else:
    # nothing to export return
    return None, materials, material_file
  try:
    out = open(filename,"w")
  except IOError:
    print 'cannot open file %s' % filename
    return None, materials, material_file
  with out:
    out.write("# Visual Components OBJ Exporter v0.1c - (c)2018 Visual Components\n")
    out.write("# File Created: %s\n\n" % datetime.datetime.now().isoformat())
    out.write("mtllib %s\n\n" % ("VC_to_Blender_materials.mtl"))
    ii = 1
    mp = vcMatrix.new()#n.WorldPositionMatrix
    mo = vcMatrix.new(mp)
    mo.P = vcVector.new()
    aaa = 0

    geosets_by_material = defaultdict(list)
    for gs in node.Geometry.GeometrySets:
      if gs.Type == VC_TRIANGLESET:
        if not gs.Material:
          material_name = 'orange'
        else:
          material_name = gs.Material.Name
        geosets_by_material[material_name].append(gs)
    for material_name, geoset_list in geosets_by_material.iteritems():
      #if gs.Material and gs.Material.Name == 'cyan':
      #  continue
      aaa += 1

      material = app.findMaterial(material_name)
      materials[material_name] = material
      new_materials[material_name] = material
      out.write("#\n# object %s\n#\n\n" % material_name)
      vertex_string = ''
      text_coord_string = ''
      face_string = ''
      vertex_indexer = 0
      for gs in geoset_list:
        for vec in zip(gs.PositionTable[::3],gs.PositionTable[1::3],gs.PositionTable[2::3]):
          v = mp*vcVector.new(vec[0]*scale,vec[1]*scale,vec[2]*scale)
          vertex_string += "v %.4f %.4f %.4f\n" % (v.X,v.Y,v.Z)
        #out.write("# %d vertices\n\n" % gs.PointCount )
        if len(gs.TextureCoordinateTable) > 0:
          for vec in zip(gs.TextureCoordinateTable[::2],gs.TextureCoordinateTable[1::2]):
            text_coord_string += "vt %.4f %.4f 0.0000\n" % vec
        else:
          for i in range(gs.PointCount):
            text_coord_string += "vt 0.0000 0.0000 0.0000\n"
        #out.write("# %d texture coords\n\n" % gs.PointCount )
        tt = gs.TriangleTable
        for v in zip(tt[::3],tt[1::3],tt[2::3]):
          face_string += "f %d/%d/%d %d/%d/%d %d/%d/%d\n" % (v[0]+ii,v[0]+ii,v[0]+ii,v[1]+ii,v[1]+ii,v[1]+ii,v[2]+ii,v[2]+ii,v[2]+ii)
        #out.write("# %d polygons\n\n" % gs.TriangleCount )
        ii += gs.PointCount
      out.write(vertex_string)
      out.write(text_coord_string)
      out.write("g %s_%i\n" % ('geo_'+material_name, aaa))
      out.write("usemtl %s\n" % sanitizefilename(material_name))
      out.write(face_string)


  # # # # # # # # # # # # # # # # # # # # #
  try:
    if material_file:
      out = open(material_file.name, 'a')
    else:
      out = open(mtlfilename,"w")
      out.write("# Visual Components OBJ like material exporter - (c)2018 Visual Components\n")
      out.write("# File Created: %s\n\n" % datetime.datetime.now().isoformat())
  except IOError:
    print 'cannot open file %s' % filename
    return None, materials, material_file
  with out:
    # # # # # # # # # # # # # # # # # # # # 
    # # # # # # # Materials # # # # # # # # 
    # # # # # # # # # # # # # # # # # # # # 

    for mn, mat in new_materials.iteritems():
      table_material = False
      if mat_mode == 'Read From Table' and mat.Name in mat_dict:
        table_material = True
        R = float(mat_dict[mat.Name]['R'])#**(1/2.2)
        G = float(mat_dict[mat.Name]['G'])#**(1/2.2)
        B = float(mat_dict[mat.Name]['B'])#**(1/2.2)
        metallic = float(mat_dict[mat.Name]['metallic'])
        roughness = float(mat_dict[mat.Name]['roughness'])
        bumpiness = float(mat_dict[mat.Name]['bumpiness'])
        clearcoat = float(mat_dict[mat.Name]['clearcoat'])
        clearcoat_rouhness = mat_dict[mat.Name]['clearcoat_rouhness'] # not implemented
        opacity = float(mat_dict[mat.Name]['opacity'])
        custom_data = mat_dict[mat.Name]['custom_data']
        #procedural_displacement = mat_dict[mat.Name]['procedural_displacement']
        #displacement_value = float(mat_dict[mat.Name]['displacement_value'])
        #displacement_scale = float(mat_dict[mat.Name]['displacement_scale'])
        color_map = mat_dict[mat.Name]['color_map']
        normal_map = mat_dict[mat.Name]['normal_map']
      out.write("newmtl %s\n" % sanitizefilename(mn))
      if table_material:
        out.write("\td %.4f\n" % opacity)
      elif mat.OpacityType == VC_MATERIAL_TRANSPARENCY_CONSTANT:
          out.write("\td %.4f\n" % mat.Opacity)
          opacity = mat.Opacity
      elif mat.OpacityType == VC_MATERIAL_TRANSPARENCY_NONE:
        out.write("\td 1.0000\n")
        opacity = 1.0
      out.write("\tKa %.4f %.4f %.4f\n" % (mat.Ambient.X,mat.Ambient.Y,mat.Ambient.Z))
      if table_material:
        out.write("\tKd %.4f %.4f %.4f\n" % (R**(1/2.2),G**(1/2.2),B**(1/2.2))) # HUOM! DOUBLE CHECK THE GAMMA CORRECTION
      else:
        try:
          out.write("\tKd %.4f %.4f %.4f\n" % (mat.BaseColor.X,mat.BaseColor.Y,mat.BaseColor.Z))
        except:
          out.write("\tKd %.4f %.4f %.4f\n" % (mat.Diffuse.X,mat.Diffuse.Y,mat.Diffuse.Z))
      out.write("\tKs %.4f %.4f %.4f\n" % (mat.Specular.X,mat.Specular.Y,mat.Specular.Z))
      out.write("\tillum 3\n")
      texturefilename = ''
      if False and table_material and color_map: #FALSE FORCED TO DISABLE 
        out.write("\tmap_Kd %s\n" % sanitizefilename(color_map))
        # MAP NEEDS TO BE COPIEND INT HE TEMPDATA FOLDER  !! NOT IMPLEMENTED YET HUOM!!
      elif mat.Texture:
        tname = basename + '_' + mn + ".png"
        tname = sanitizefilename(tname)
        texturefilename = "file:///" + os.path.dirname(filename) +'\\'+ tname
        app.saveBitmap(mat.Texture, texturefilename)
        out.write("\tmap_Kd %s\n" % tname)
      #
      if False and table_material and normal_map: #FALSE FORCED TO DISABLE 
        out.write("\tmap_bump %s\n" % sanitizefilename(normal_map))
        # MAP NEEDS TO BE COPIEND INT HE TEMPDATA FOLDER  !! NOT IMPLEMENTED YET HUOM!!
      else:
        try:
          mat.BumpMap
          bumpmapAvailable = True
        except:
          bumpmapAvailable = False
        bumptexturefilename = ''
        if bumpmapAvailable:
          tname = basename + '_' + mn + '_BUMP' + ".png"
          tname = sanitizefilename(tname)
          bumptexturefilename = "file:///" + os.path.dirname(filename) +'\\'+ tname
          app.saveBitmap(mat.BumpMap, bumptexturefilename)
          #
          out.write("\tmap_bump %s\n" % tname)
      #
      if table_material:
        out.write("\tbumpiness %f\n" % bumpiness)
      else:
        try:
          bumpiness = mat.Bumbiness #typo in API should be bumpiness
        except:
          bumpiness = 0.0
        out.write("\tbumpiness %f\n" % bumpiness)
      #
      if table_material:
        out.write("\tmetallic %f\n" % metallic)
      else:
        try:
          metallic = mat.Metallic
        except:
          metallic = 0.0
        out.write("\tmetallic %f\n" % metallic)
      # clearcoat
      if table_material:
        out.write("\tclearcoat %f\n" % clearcoat)
      else:
        try:
          clearcoat = mat.Clearcoat
        except:
          clearcoat = 0.0
        out.write("\tclearcoat %f\n" % clearcoat)
      # custom_data
      if table_material:
        out.write("\tcustom_data %s\n" % custom_data)
      else:
        try:
          custom_data = mat.CustomData
        except:
          custom_data = 'N/A'
        out.write("\tcustom_data %s\n" % custom_data)
      '''
      # procedural_displacement
      if table_material:
        out.write("\tprocedural_displacement %s\n" % procedural_displacement)
      else:
        try:
          procedural_displacement = mat.ProceduralDisplacement
        except:
          procedural_displacement = 'N/A'
        out.write("\tprocedural_displacement %s\n" % procedural_displacement)
      #
      # displacement_value for procedural
      if table_material:
        out.write("\tdisplacement_value %f\n" % displacement_value)
      else:
        try:
          displacement_value = mat.DisplacementValue
        except:
          displacement_value = 0.0
        out.write("\tdisplacement_value %f\n" % displacement_value)
      #
      # displacement_scale for procedural
      if table_material:
        out.write("\tdisplacement_scale %f\n" % displacement_scale)
      else:
        try:
          displacement_scale = mat.DisplacementScale
        except:
          displacement_scale = 1.0
        out.write("\tdisplacement_scale %f\n" % displacement_scale)
      '''
      #
      if table_material:
        out.write("\troughness %f\n" % roughness)
      else:
        try:
          roughness = mat.Roughness
        except:
          roughness = 0.0
        out.write("\troughness %f\n" % roughness)
      out.write("\n")
      if mat_mode == 'Write To Table' and mat_csv:
        mat_csv.write(mat.Name + SEP)
        mat_csv.write(str(mat.Diffuse.X) + SEP)
        mat_csv.write(str(mat.Diffuse.Y) + SEP)
        mat_csv.write(str(mat.Diffuse.Z) + SEP)
        mat_csv.write(str(metallic) + SEP)
        mat_csv.write(str(roughness) + SEP)
        mat_csv.write(str(bumpiness) + SEP)
        mat_csv.write(str(clearcoat) + SEP)
        mat_csv.write('' + SEP)
        mat_csv.write(str(opacity) + SEP)
        mat_csv.write(custom_data + SEP)
        #mat_csv.write(procedural_displacement + SEP)
        #mat_csv.write(str(displacement_value) + SEP)
        #mat_csv.write(str(displacement_scale) + SEP)
        mat_csv.write(texturefilename + SEP)
        mat_csv.write(bumptexturefilename + LINECHANGE)
        #material_table_csv.close()
  return True, materials, out


